from dataclasses import dataclass
import os
import traceback
from google.cloud import speech
from google.cloud import storage

from owoi_audio_to_clip.WordTimestamp import WordTimestamp


@dataclass
class TranscriptFactory:
    gcs_uri: str
    language_code: str = "en-US"
    word_timestamps: list[WordTimestamp] = []
    speech_client = speech.SpeechClient()
    storage_client = storage.Client()
    google_credentials_key: str = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")

    def _get_recognition_config(self):
        return speech.RecognitionConfig(
            encoding=speech.RecognitionConfig.AudioEncoding.LINEAR16,
            language_code=self.language_code,
            sample_rate_hertz=44100,
            audio_channel_count=2,
            enable_word_time_offsets=True,
            use_enhanced=True,
            model="video",
        )

    def transcribe_audio_to_text(self) -> list[WordTimestamp]:
        self.word_timestamps = []
        try:
            operation = self.speech_client.long_running_recognize(
                config=self._get_recognition_config(),
                audio=speech.RecognitionAudio(uri=self.gcs_uri),
            )
            response = operation.result()

            for result in response.results:
                alternative = result.alternatives[0]
                for word_info in alternative.words:
                    self.word_timestamps.append(
                        WordTimestamp(
                            word_info.word, word_info.start_time, word_info.end_time
                        )
                    )
            return self.word_timestamps
        except:
            traceback.print_exc()
            raise Exception("Could not transcribe audio to text")

    def get_word_timestamps(self):
        return self.word_timestamps



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['owoi_audio_to_clip']

package_data = \
{'': ['*']}

install_requires = \
['Google-Images-Search>=1.4.6,<2.0.0',
 'black>=22.10.0,<23.0.0',
 'google-cloud-speech>=2.16.2,<3.0.0',
 'google-cloud-storage>=2.5.0,<3.0.0',
 'moviepy>=1.0.3,<2.0.0']

setup_kwargs = {
    'name': 'owoi-audiotoclip',
    'version': '0.1.1',
    'description': 'Package to convert audio to video with google speech to text and google image search',
    'long_description': '# OWOI_AudioToClip\nPython module used for the school project OWOI (One Word One Image)\n\n## Installation\n\nAfter git cloning the repository, you can install the dependencies with the following command:\n\n```bash\npoetry install\n```\n\n## Credentials\n\nPlease provide your credentials in the following environment variables:\n\n```bash\nexport GOOGLE_APPLICATION_CREDENTIALS="path/to/credentials.json"\nexport GOOGLE_IMAGES_SEARCH_TOKEN="token"\nexport GOOGLE_SEARCH_ID="id"\n```\n\n## Classes\n\n### TranscriptFactory\n\nThis class is used to create a transcript from a text file. It will create a list of words and a list of timestamps.\n\n```python\t\nfrom audio_to_clip import ClipMakerFactory\n\ntranscript_factory = TranscriptFactory(gcs_uri="gs://bucket/file.mp3")\n```\n\nMethods:\n- ***transcribe_audio_to_text() -> list[dict]***: transcribe audio to text from the gcs_uri and returns a list of dict with the following keys: "word", "start_time" and "end_time"\n- ***get_word_timestamps() -> list[dict]***: returns a list of dict with the following keys: "word", "start_time" and "end_time"\n\nThis Class should be used to create a transcript from a text file before creating a clip with the ClipMakerFactory.\n\n### ClipMakerFactory\n\nThis class is used to create a clip from a transcript.\n\n```python\nfrom audio_to_clip import ClipMakerFactory\n\nclip_makfer_factory = ClipMagerFactory(video_name, username, transcript, gcs_bucket_dest, local_storage_dest, gcs_bucket_audio)\n```\n\nParams:\n- ***video_name***:str -> name of the video\n- ***username***:str -> name of the user\n- ***transcript***:list[WordTimestamp] -> list of WordTimestamp\n- ***gcs_bucket_dest***:str -> name of the gcs bucket destination\n- ***local_storage_dest***:str -> path to the local storage destination\n- ***gcs_bucket_audio***:str -> name of the gcs bucket where the audio is stored\n\nMethods:\n- ***clip_maker(word_timestamps: list[WordTimestamp]) -> VideoFileClip***: creates a clip from the transcript and returns a VideoFileClip\n- ***upload_video_to_gcs() -> None***: uploads the video to the gcs bucket with the name: `username/video_name.mp4`, should be called after clip_maker',
    'author': 'Pierre-Louis Sergent',
    'author_email': 'papa.louis59@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
import os
import traceback
import requests
from moviepy.editor import AudioFileClip, VideoFileClip, ImageClip
from google.cloud import storage
from google_images_search import GoogleImagesSearch

from owoi_audio_to_clip.WordTimestamp import WordTimestamp


@dataclass
class ClipMakerFactory:
    video_name: str
    username: str
    words_timestamps: list[WordTimestamp]
    gcs_bucket_dest: str
    local_dest: str
    gcs_bucket_audio: str
    audio_file_clip: AudioFileClip = None
    video_file_clip: VideoFileClip = None
    images: list[str] = []
    storage_client = storage.Client()
    google_images_search_token = os.environ.get("GOOGLE_IMAGES_SEARCH_TOKEN")
    google_search_id = os.environ.get("GOOGLE_SEARCH_ID")
    google_credentials_key: str = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")

    def _get_audio_file(self, filename: str) -> AudioFileClip:
        if self.audio_file_clip is not None:
            return self.audio_file_clip
        try:
            os.makedirs("./tmp/", exist_ok=True)
            bucket = self.storage_client.bucket(self.gcs_bucket_audio)
            blob = bucket.blob(filename)
            blob.download_to_filename("./tmp/" + filename)
            return AudioFileClip("./tmp/" + filename)
        except:
            traceback.print_exc()
            raise Exception("Could not download audio file from GCS")
    
    def clip_maker(self, word_timestamps: list[WordTimestamp]) -> VideoFileClip:
        try:
            for wordts in word_timestamps:
                self.images.append(wordts.get_word_dict())
            
            clips = [self._define_image_clip(img_dict) for img_dict in self.images]
            self.video_file_clip = VideoFileClip.concatenate_videoclips(clips)
            self.video_file_clip.write_videofile(
                self.local_dest + self.video_name + ".mp4", fps=24, codec="mpeg4"
            )
            self.video_file_clip.set_audio(self._get_audio_file(self.video_name + ".mp3"))
            return self.video_file_clip
        except:
            traceback.print_exc()
            raise Exception("Could not create video file clip")
    
    def _get_image_url_from_google_image_search(self, word: str) -> str:
        gis = GoogleImagesSearch(self.google_images_search_token, self.google_search_id)
        _search_params = {
            'q': word,
            'num': 1,
            'safe': 'off',
            'fileType': 'png',
            'imgType': 'photo',
            'imgSize': 'MEDIUM',
            'rights': 'cc_publicdomain'
        }
        gis.search(search_params=_search_params)
        return gis.results()[0].url
    
    def _define_image_clip(self, img_dict: dict) -> ImageClip:
        url = self._get_image_url_from_google_image_search(img_dict['word'])
        response = requests.get(url)
        open(self.local_dest + img_dict['word'] + '.png', 'wb').write(response.content)
        return ImageClip(
                    self.local_dest + img_dict['word'] + '.png'
                ).set_duration(img_dict['end_time'] - img_dict['start_time'])
    
    def upload_video_to_gcs(self):
        bucket = self.storage_client.bucket(self.gcs_bucket_dest)
        blob = bucket.blob(f"{self.username}/{self.video_name}.mp4")
        blob.upload_from_filename(f"{self.local_dest}{self.username}/{self.video_name}.mp4")

    def get_video_file(self) -> VideoFileClip:
        if self.video_file_clip is not None:
            return self.video_file_clip



from dataclasses import dataclass

@dataclass
class WordTimestamp:
    word: str
    start: float
    end: float

    def __str__(self) -> str:
        return f"Word: {self.word}, start: {self.start}, end: {self.end}"

    def __repr__(self) -> str:
        return f"WordTimestamp(word={self.word}, start={self.start}, end={self.end})"

    def get_word_dict(self) -> dict:
        return {"word": self.word, "start": self.start, "end": self.end}
